package com.mr.test;

import java.util.Random;

import android.text.TextUtils;

/**
 * 随机数工具类
 */
public class RandomUtils {

	public static final String NUMBERS_AND_LETTERS = "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
	public static final String NUMBERS = "0123456789";
	public static final String LETTERS = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
	public static final String CAPITAL_LETTERS = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
	public static final String LOWER_CASE_LETTERS = "abcdefghijklmnopqrstuvwxyz";

	/**
	 * 得到一个固定长度的随机字符串（大写,小写字母和数字组成）
	 * 
	 * @param length
	 */
	public static String getRandomNumbersAndLetters(int length) {
		return getRandom(NUMBERS_AND_LETTERS, length);
	}

	/**
	 * 得到一个固定长度的随机字符串（数字组成）
	 * 
	 * @param length
	 */
	public static String getRandomNumbers(int length) {
		return getRandom(NUMBERS, length);
	}

	/**
	 * 得到一个固定长度的随机字符串（大写和小写字母组成）
	 * 
	 * @param length
	 */
	public static String getRandomLetters(int length) {
		return getRandom(LETTERS, length);
	}

	/**
	 * 得到一个固定长度的随机字符串（大写字母组成）
	 * 
	 * @param length
	 */
	public static String getRandomCapitalLetters(int length) {
		return getRandom(CAPITAL_LETTERS, length);
	}

	/**
	 * 得到一个固定长度的随机字符串（小写字母组成）
	 * 
	 * @param length
	 */
	public static String getRandomLowerCaseLetters(int length) {
		return getRandom(LOWER_CASE_LETTERS, length);
	}

	/**
	 * 得到一个固定长度的随机字符串（ String source 组成）
	 * 
	 * @param source
	 * @param length
	 */
	public static String getRandom(String source, int length) {
		return TextUtils.isEmpty(source) ? null : getRandom(source.toCharArray(), length);
	}

	/**
	 * 得到一个固定长度的随机字符串（ char[] sourceChar 组成）
	 * 
	 * @param sourceChar
	 * @param length
	 */
	public static String getRandom(char[] sourceChar, int length) {
		if (sourceChar == null || sourceChar.length == 0 || length < 0) {
			return null;
		}

		StringBuilder str = new StringBuilder(length);
		Random random = new Random();
		for (int i = 0; i < length; i++) {
			str.append(sourceChar[random.nextInt(sourceChar.length)]);
		}
		return str.toString();
	}

	/**
	 * 得到随机 int 介于 0 和 max
	 * 
	 * @param max
	 */
	public static int getRandom(int max) {
		return getRandom(0, max);
	}

	/**
	 * 得到随机 int 介于 min 和 max
	 * 
	 * @param min
	 * @param max
	 */
	public static int getRandom(int min, int max) {
		if (min > max) {
			return 0;
		}
		if (min == max) {
			return min;
		}
		return min + new Random().nextInt(max - min);
	}

	/**
	 * 洗牌算法,随机排列指定的数组（使用默认的随机性）
	 * 
	 * @param objArray
	 */
	public static boolean shuffle(Object[] objArray) {
		if (objArray == null) {
			return false;
		}

		return shuffle(objArray, getRandom(objArray.length));
	}

	/**
	 * 洗牌算法,随机排列指定的数组
	 * 
	 * @param objArray
	 * @param shuffleCount
	 */
	public static boolean shuffle(Object[] objArray, int shuffleCount) {
		int length;
		if (objArray == null || shuffleCount < 0 || (length = objArray.length) < shuffleCount) {
			return false;
		}

		for (int i = 1; i <= shuffleCount; i++) {
			int random = getRandom(length - i);
			Object temp = objArray[length - i];
			objArray[length - i] = objArray[random];
			objArray[random] = temp;
		}
		return true;
	}

	/**
	 * 洗牌算法,随机排列指定int数组（使用默认的随机性）
	 * 
	 * @param intArray
	 */
	public static int[] shuffle(int[] intArray) {
		if (intArray == null) {
			return null;
		}

		return shuffle(intArray, getRandom(intArray.length));
	}

	/**
	 * 洗牌算法,随机排列指定int数组
	 * 
	 * @param intArray
	 * @param shuffleCount
	 */
	public static int[] shuffle(int[] intArray, int shuffleCount) {
		int length;
		if (intArray == null || shuffleCount < 0 || (length = intArray.length) < shuffleCount) {
			return null;
		}

		int[] out = new int[shuffleCount];
		for (int i = 1; i <= shuffleCount; i++) {
			int random = getRandom(length - i);
			out[i - 1] = intArray[random];
			int temp = intArray[length - i];
			intArray[length - i] = intArray[random];
			intArray[random] = temp;
		}
		return out;
	}
}